/*
 * Copyright (C) 2000-2007 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

/*!
 * $Log: access-node.c,v $
 * Revision 1.2  2007/01/17 15:29:45  dyerm
 * code clean-up
 *
 */

#include "../defs.h"
#include "program_version.h"
#include <debug/toolbox.h>

THREAD(coninfoLoop, arg){
	while(1){
		NutSleep(10000);
		btn_terminal_process_cmd("coninfo");
	}
}

bt_acl_pkt_buf* sensor_data_cb(bt_acl_pkt_buf* pkt_buf,
									u_char* data,
									u_short data_len,
									u_short service_nr,
									void* cb_arg){
	u_char* source = mhop_cl_get_source_addr(pkt_buf->pkt);
	sensor_data_t* sensor_data = (sensor_data_t*) data;
	printf(":S "SADDR_FMT" %u %d %u\n", SADDR(source),
				sensor_data->mic,
				sensor_data->light,
				sensor_data->temp);
	
	// let all LED's blink
	btn_led_add_pattern(BTN_LED_PATTERN_ON_OFF, 2, 1, 1);
	
	// free the received message
	return pkt_buf;
}


int main(void)
{
    // hardware init
    btn_hardware_init();
    btn_led_init(1);

    // init terminal app uart
    u_long baud = 57600;     // serial baud rate
    NutRegisterDevice(&APP_UART, 0, 0);
    freopen(APP_UART.dev_name, "r+", stdout);
    _ioctl(_fileno(stdout), UART_SETSPEED, &baud);
    btn_terminal_init(stdout, "[access]$");

    // hello message
    printf("\n# ------------------------------------------------");
    printf("\n# Welcome to the EWSN 2007 Tutorial (c) ETH Zurich\n");
    printf("# program version: %s\n", PROGRAM_VERSION);
    printf("# ------------------------------------------------");

    printf("\nbooting bluetooth module... ");
    // bluetooth module on (takes a while)
    btn_hardware_bt_on();
    printf("ok.\n\r");
    
    // Start bt-stack and let the initialization begin
    printf("init bt-stack... ");
	struct btstack* bt_stack;
    bt_stack = bt_hci_init(&BT_UART);
    printf("done.\n");

	// Initialize connection-less multi-hop layer
    // Set ACL packet types
    printf("setting acl pkt types... ");
    bt_acl_init(bt_stack, BT_HCI_PACKET_TYPE_DM3);
    printf("done.\n");
    
    // Init protocol/service multiplexor
    printf("init protcol/service mux... ");
	bt_psm_t* psmux;
    psmux = bt_psm_init(bt_stack, MAX_NR_SERVICES, NR_BUFFERS);
    printf("done.\n");
    
    // Init connectionless l2cap stack
    printf("init connectionless l2cap... ");
    l2cap_cl_init(bt_stack, psmux);
    printf("done.\n");
    
    // Init connection manager
    printf("init connection manager... ");
    con_mgr_init(bt_stack, psmux, CM_PSM, bt_hci_register_con_table_cb, CM_COD);
    printf("done.\n");
    
    // Init connectionless multihop protocol
    printf("init connectionless multi-hop protocol... ");
    mhop_cl_init(bt_stack, psmux, MHOP_PSM, NR_BUFFERS, con_mgr_register_con_table_cb);
    rpc_init(psmux, 8, RPC_PROC_PSM, RPC_RESULT_PSM);
    printf("done.\n");
	   
	// register service at the service / protocol multiplexor
	bt_psm_service_register(psmux, SENSO_PSM, sensor_data_cb, NULL);

	// init terminal & give hint
    mhop_con_info_init();

   	NutThreadCreate("T_coninfo", coninfoLoop, 0, 256);    
		
    // terminal mode
    btn_terminal_run(BTN_TERMINAL_NOFORK, 0);

    return 0;
}
