#ifndef MHOP_CL_DEFS_H_
#define MHOP_CL_DEFS_H_

#include <mhop/mhop_cl.h>
// TODO: this include is only needed because of the definition of the psm
// field size, which is defined in <bt/l2cap_cl.h>. This is not good: it
// should be defined in <bt/bt_psm.h>
#include <bt/l2cap_cl.h>

/// maximal size of the forwarding table
#define MHOP_CL_MAX_FWD_TABLE_SIZE				10

#define MHOP_CL_MAX_NR_SERVICES					8

/// paramater for avoiding multiple reception of broadcast packets
#define MHOP_CL_BC_WINDOW 64

#define BT_HCI_CON_HANDLE_OWN 0

#define MHOP_CL_PKT_TYPE_UNICAST		0
#define MHOP_CL_PKT_TYPE_BROADCAST		1
#define MHOP_CL_PKT_TYPE_SRC_DEL		2


/** 
 * \brief Definition of the multi-hop control header.
 * 
 * The information fields of the multi-hop control header are required for
 * routing data properly to a remote device via several intermediate hops.
 * \see mhop_cl_send_pkt_to()
 * 
 \verbatim
 -------------------------------------------------------------------
 | type | seq | ttl | hop_cnt |  source   | destination | dest_psm |
 -------------------------------------------------------------------
 .  1      1     1       1     BD_ADDR_LEN  BD_ADDR_LEN      2
 \endverbatim
 */
typedef struct _mhop_cl_pkt_hdr {
	u_char type;				///< flags, defining the type of the packet
    u_char seq;                 ///< broadcast sequence number
    u_char ttl;					///< time to live (0 = inf)
    u_char hop_cnt;             ///< hop count (distance to host)
    bt_addr_t source;           ///< address of the sending node
    bt_addr_t destination;      ///< address of the destination node
    u_char psm[L2CAP_CL_PSM_FIELD_SIZE];
    							///< PSM the packet will be delivered to on
    							///< the receiving side
} mhop_cl_pkt_hdr;

/**
 * \brief connection-less multi-hop stack.
 */
typedef struct _mhop_cl_stack_s {
	bt_addr_t my_addr;          ///< own bt addr
	struct btstack* bt_stack;	///< Pointer to the lower layer l2cap cl stack
    bt_psm_t* psmux;			///< Pointer to the protocol / service multiplexor
	u_short mhop_psm;			///< PSM of the multi-hop cl service
    mhop_cl_fwt_entry fwt[MHOP_CL_MAX_FWD_TABLE_SIZE];
    bt_psm_t* fwd_mux;
    u_char mylastseq;           ///< last sequence number I(!) used
    bt_hci_con_handle_t reliable_con_handles[BT_HCI_MAX_NUM_CON];
    							///< list with reliable connection handles
    HCI_CON_TABLE_CB;
    void* con_table_cb_arg;
    MHOP_CL_FWTABLE_CB;			///< Forwarding table change callback
    void* fwt_cb_arg;			///< Forwarding table change callback argument
    							///< forwarding table
} _mhop_cl_stack_t;

#endif /*MHOP_CL_DEFS_H_*/
